/**
 * Created by Dukobpa3 on 12/27/13.
 */
package patterncraft.raven
{
	import patterncraft.tank.*;
	import flash.events.Event;

	internal class RavenContext
	{
		private var _currentState:String;
		private var _pendingState:String;

		private var _statesMap:Object;
		private var _transitionsMap:Object;

		public function RavenContext(raven:Raven)
		{
			// Заполним вспомогательній словарь стейтов по названию
			_statesMap = {};
			_statesMap[Raven.STATE_FLY] = new FlyState(raven);
			_statesMap[Raven.STATE_SWIM] = new SwimState(raven);

			// Зарегистрируем доступные переходы
			// Получится такая вот цепочка переходов: siege <--> tank <--> speed
			_transitionsMap = {};
			_transitionsMap[Raven.STATE_FLY] = [Raven.STATE_SWIM];
			_transitionsMap[Raven.STATE_SWIM] = [Raven.STATE_FLY];

			// стартовое состояние
			_currentState = Raven.STATE_FLY;
			state.apply();
		}

		public function setState(state:String):void {

			// Если пытаемся установить состояние которое уже установлено
			if (state == _currentState) {
				trace("Already in " + state + " Mode!");
				return;
			}
			// Либо сейчас происходит транзишн - просто ничего не делаем.
			if (_currentState == SiegeTank.STATE_NONE) {
				trace("Previous transition not complete!");
				return;
			}

			// Если в списке доступных транзишинов нету запрошенного
			if(_transitionsMap[_currentState].indexOf(state) == -1) {
				trace("Can't go to "+ state + " from " + _currentState);
				return;
			}

			// Запускаем транзишн. Подписываемся на его окончание
			_statesMap[_currentState].addEventListener(Event.COMPLETE, onTransitionComplete);
			_statesMap[_currentState].transitionTo(_pendingState);


			_currentState = SiegeTank.STATE_NONE; // текущий ставим как транзишн
			_pendingState = state; // ставим ожидаемый стейт.
		}

		private function onTransitionComplete(event:Event):void {

			trace("transition complete");
			event.currentTarget.removeEventListener(Event.COMPLETE, onTransitionComplete);

			// Отмечаемся в самих переменных стейтов.
			_currentState = _pendingState;
			_pendingState = "";

			// По завершении транзишина ставим настройки нового стейта
			state.apply();
		}

		public function get state():IRavenState { return _statesMap[_currentState]; }
	}
}
