/**
 * Created by Dukobpa3 on 12/27/13.
 */
package patterncraft.tank
{
	import flash.events.Event;

	internal class TankContext
	{
		private var _currentState:String;
		private var _pendingState:String;

		private var _statesMap:Object;
		private var _transitionsMap:Object;

		public function TankContext(tank:SiegeTank)
		{
			// Заполним вспомогательній словарь стейтов по названию
			_statesMap = {};
			_statesMap[SiegeTank.STATE_TANK] = new TankState(tank);
			_statesMap[SiegeTank.STATE_SIEGE] = new SiegeState(tank);
			_statesMap[SiegeTank.STATE_SPEED] = new SpeedState(tank);

			// Зарегистрируем доступные переходы
			// Получится такая вот цепочка переходов: siege <--> tank <--> speed
			_transitionsMap = {};
			// Из состояния танка можно перейти в любые два других
			_transitionsMap[SiegeTank.STATE_TANK] = [SiegeTank.STATE_SIEGE, SiegeTank.STATE_SPEED];
			// Из осадного или скоростного режима можно только в состояние танка
			_transitionsMap[SiegeTank.STATE_SIEGE] = [SiegeTank.STATE_TANK];
			_transitionsMap[SiegeTank.STATE_SPEED] = [SiegeTank.STATE_TANK];

			// стартовое состояние
			_currentState = SiegeTank.STATE_TANK;
			state.apply();
		}

		public function setState(state:String):void {

			// Если пытаемся установить состояние которое уже установлено
			if (state == _currentState) {
				trace("Already in " + state + " Mode!");
				return;
			}
			// Либо сейчас происходит транзишн - просто ничего не делаем.
			if (_currentState == SiegeTank.STATE_NONE) {
				trace("Previous transition not complete!");
				return;
			}

			// Если в списке доступных транзишинов нету запрошенного
			if(_transitionsMap[_currentState].indexOf(state) == -1) {
				trace("Can't go to "+ state + " from " + _currentState);
				return;
			}

			// Запускаем транзишн. Подписываемся на его окончание
			_statesMap[_currentState].addEventListener(Event.COMPLETE, onTransitionComplete);
			_statesMap[_currentState].transitionTo(_pendingState);


			_currentState = SiegeTank.STATE_NONE; // текущий ставим как транзишн
			_pendingState = state; // ставим ожидаемый стейт.
		}

		private function onTransitionComplete(event:Event):void {

			trace("transition complete");
			event.currentTarget.removeEventListener(Event.COMPLETE, onTransitionComplete);

			// Отмечаемся в самих переменных стейтов.
			_currentState = _pendingState;
			_pendingState = "";

			// По завершении транзишина ставим настройки нового стейта
			state.apply();
		}

		public function get state():ISiegeTankState { return _statesMap[_currentState]; }
	}
}
